@@ -24,7 +24,7 @@ from .utils.event import connect, unconnect, emit
 
 __author__ = 'Cyrille Rossant'
 __email__ = 'cyrille.rossant at gmail.com'
-__version__ = '2.4.1'
+__version__ = '2.4.2'
 __version_git__ = __version__ + _git_version()
 
 
@@ -39,13 +39,15 @@ _logger_date_fmt = '%H:%M:%S'
 
 
 class _Formatter(logging.Formatter):
+    color_codes = {'L': '94', 'D': '90', 'I': '0', 'W': '33', 'E': '31'}
+
     def format(self, record):
         # Only keep the first character in the level name.
         record.levelname = record.levelname[0]
         filename = op.splitext(op.basename(record.pathname))[0]
         record.caller = '{:s}:{:d}'.format(filename, record.lineno).ljust(20)
         message = super(_Formatter, self).format(record)
-        color_code = {'D': '90', 'I': '0', 'W': '33', 'E': '31'}.get(record.levelname, '7')
+        color_code = self.color_codes.get(record.levelname, '90')
         message = '\33[%sm%s\33[0m' % (color_code, message)
         return message
 
