@@ -152,15 +152,17 @@ def _apply_op(op, arg, arr):
     return f(arg) if arg is not None else f()
 
 
-def _memmap_flat(path, dtype=None, n_channels=None, offset=0):
+def _memmap_flat(path, dtype=None, n_channels=None, offset=0, mode='r+'):
     path = Path(path)
     # Find the number of samples.
     assert n_channels > 0
     fsize = path.stat().st_size
     item_size = np.dtype(dtype).itemsize
+    assert (fsize - offset) % (item_size * n_channels) == 0,\
+        'Inconsistent number of channels between the params file and the binary dat file'
     n_samples = (fsize - offset) // (item_size * n_channels)
     shape = (n_samples, n_channels)
-    return np.memmap(path, dtype=dtype, offset=offset, shape=shape)
+    return np.memmap(path, dtype=dtype, offset=offset, shape=shape, mode=mode)
 
 
 #------------------------------------------------------------------------------
@@ -303,7 +305,8 @@ class BaseEphysReader(object):
 
 
 class FlatEphysReader(BaseEphysReader):
-    def __init__(self, paths, sample_rate=None, dtype=None, offset=0, n_channels=None, **kwargs):
+    def __init__(self, paths, sample_rate=None, dtype=None, offset=0, n_channels=None, mode='r',
+                 **kwargs):
         super(FlatEphysReader, self).__init__()
         if isinstance(paths, (str, Path)):
             paths = [paths]
@@ -312,7 +315,7 @@ class FlatEphysReader(BaseEphysReader):
         self.name = paths[0].stem
         self.dir_path = paths[0].parent
         self._mmaps = [
-            _memmap_flat(path, dtype=dtype, n_channels=n_channels, offset=offset)
+            _memmap_flat(path, dtype=dtype, n_channels=n_channels, offset=offset, mode=mode)
             for path in paths]
 
         self.sample_rate = sample_rate
@@ -459,6 +462,7 @@ def _get_ephys_constructor(obj, **kwargs):
             return None, None, {}
         assert path.exists()
         ext = path.suffix
+        assert ext, "No extension found in file `%s`" % path
         # Mtscomp file
         if ext == '.cbin':
             reader = mtscomp.Reader(n_threads=mp.cpu_count() // 2)
@@ -471,7 +475,7 @@ def _get_ephys_constructor(obj, **kwargs):
             return (NpyEphysReader, obj, kwargs)
             # TODO: other standard binary formats
         else:  # pragma: no cover
-            raise IOError("Unknown file extension: %s.", ext)
+            raise IOError("Unknown file extension: `%s`." % ext)
     elif isinstance(obj, (tuple, list)):
         if obj:
             # Concatenate the main argument to the constructor.
